#version 330 core

// Input vertex data, different for all executions of this shader.
layout(location = 0) in vec3 vPosition; // vertex position (in model space)
layout(location = 1) in vec3 vNormal; // vertex normal (in model space)
layout(location = 2) in vec2 vTexCoord; // texture coordinate (in model space)

// Output data will be interpolated for each fragment.
out vec2 TexCoordPass;
out vec3 PositionWorldPass;
out vec3 NormalViewPass;
out vec3 EyeDirectionViewPass;
out vec3 LightDirectionViewPass;

// Values that stay constant for the whole mesh.
uniform mat4 gModel, gView, gProjection;
uniform vec4 gLightPosition; // light (in world)
uniform vec3 gEyePosition; // camera (in world)

void main()
{
	// Output position of the vertex, in clip space : gMVP * position
	mat4 MVP = gProjection * gView * gModel;
	gl_Position =  MVP * vec4(vPosition,1);
	
	// Position of the vertex, in worldspace : gWorld * position
	PositionWorldPass = (gModel * vec4(vPosition,1)).xyz;
	
	// Vector that goes from the vertex to the camera, in camera space.
	// In camera space, the camera is at the origin (0,0,0).
	vec3 PositionView = (gView * gModel * vec4(vPosition,1)).xyz;
	EyeDirectionViewPass = gEyePosition - PositionView;

	// Vector that goes from the vertex to the light, in camera space. M is ommited because it's identity.
	vec3 LightPositionView = (gView * gLightPosition).xyz;
	LightDirectionViewPass = LightPositionView + EyeDirectionViewPass;
	
	// Normal of the the vertex, in camera space
	NormalViewPass = (gView * gModel * vec4(vNormal,0)).xyz; // Only correct if ModelMatrix does not scale the model ! Use its inverse transpose if not.
	
	// UV of the vertex. No special space for this one.
	TexCoordPass = vTexCoord;
}

