#version 330 core

// Interpolated values from the vertex shaders
in vec2 TexCoordPass;
in vec3 PositionWorldPass;
in vec3 NormalViewPass;
in vec3 EyeDirectionViewPass;
in vec3 LightDirectionViewPass;

// Ouput data
out vec4 Color;

// Values that stay constant for the whole mesh.
uniform sampler2D gTextureSampler;

uniform mat4 gView;
uniform mat4 gModel;
uniform vec4 gLightPosition; // light (in world)
uniform vec4 gMaterialAmbientColor;
uniform vec4 gMaterialDiffuseColor;
uniform vec4 gMaterialSpecularColor;
uniform float gSpecularShininess;

void main()
{
	// Light emission properties
	// You probably want to put them as uniforms
	vec4 LightColor = vec4(1,1,1,1);
	float LightPower = 50.0f;
	
	// Material properties
	vec4 MaterialDiffuseColor = texture2D(gTextureSampler, TexCoordPass).rgba * gMaterialDiffuseColor.rgba;
	vec4 MaterialAmbientColor = gMaterialAmbientColor * MaterialDiffuseColor;
	vec4 MaterialSpecularColor = gMaterialSpecularColor;

	// Distance to the light
	float distance = length(gLightPosition.xyz - PositionWorldPass);

	// Normal of the computed fragment, in camera space
	vec3 N = normalize(NormalViewPass);
	// Direction of the light (from the fragment to the light)
	vec3 L = normalize(LightDirectionViewPass);
	// Cosine of the angle between the normal and the light direction, 
	// clamped above 0
	//  - light is at the vertical of the triangle -> 1
	//  - light is perpendicular to the triangle -> 0
	//  - light is behind the triangle -> 0
	float cosTheta = clamp(dot(N, L), 0, 1);
	
	// Eye vector (towards the camera)
	vec3 V = normalize(EyeDirectionViewPass);
	// Direction in which the triangle reflects the light
	vec3 R = reflect(-L, N);
	// Cosine of the angle between the Eye vector and the Reflect vector,
	// clamped to 0
	//  - Looking into the reflection -> 1
	//  - Looking elsewhere -> < 1
	float cosAlpha = clamp(dot(V,R), 0, 1);
	float shinness = 5;

	Color = 
		// Ambient : simulates indirect lighting
		MaterialAmbientColor +
		// Diffuse : "color" of the object
		MaterialDiffuseColor * LightColor * LightPower * cosTheta / (distance*distance) +
		// Specular : reflective highlight, like a mirror
		MaterialSpecularColor * LightColor * LightPower * pow(cosAlpha, 5) / (distance*distance);
}